#include <string>
#include <set>

#include <Tracer.h>
#include <errstr.h>
#include <Dal.h>
#include <stringutil.h>

#include "CalIndexSet.h"

#include "CalIndex.h"


static string key(CcfConstituent * c, bool master){
  PROC
  string s = c->scope() + "-" + c->type();
  master ? s += "-" + toString(c->issue()) : s;
  RETURN s;
}

CalIndex::CalIndex(const string name, const bool master , const string category )
  :  _name(name),
     _category(category),
     _master(master),
     _exists(dataSetServer->exists(name))
     
{
  PROC ENDPROC
}

void CalIndex::addConstituent(CcfConstituent * c)
{
  PROC
  if(! c->isValid())
    errstr << c->name() + " is not a valid CCF constituent. Ignored." <<
      Sas::warning("InvalidCcfConstituent");
  else {
    pair<set<string>::iterator, bool> p = _tally.insert(key(c,_master));
    if(! p.second && ! _master){
      errstr << "A constituent with this instrument-type key: " + key(c,_master) + " is already present. " + c->name() + " will be ignored." << Sas::message(Msg::AppLib, Msg::Sparse);
    } else {
      push_back(c);
    }
  }
  ENDPROC
}

bool CalIndex::removeConstituent(CcfConstituent *c)
{
  PROC
  for(CalIndex::const_iterator i = begin(); i != end(); i++)
    if(CcfConstituent::equal(*i, c)){
      remove(c);
      string ccfName =  (*i)->name();
      int val = _tally.erase(key(c,_master));
      errstr << "Removed ("<< val << ") " << ccfName << " from " << name() << Sas::message(Msg::AppLib, Msg::Sparse);
      ALTRETURNV(true)
    }
  RETURN false;
}

bool CalIndex::removeConstituent(const string type, const string scope)
{
  PROC
    errstr << "Requested to remove " << type << "/" << scope << Sas::message(Msg::AppLib, Msg::Noisy);
  for(CalIndex::const_iterator i = begin(); i != end(); i++){
    errstr << "Considering " << (*i)->type() << " " << (*i)->scope() << Sas::message(Msg::AppLib, Msg::Noisy);
    if((*i)->type() == type && (*i)->scope() == scope)
      ALTRETURNV(removeConstituent(*i));
  }
  RETURN false;
}

    
void CalIndex::addConstituentsFromSet(const string set)
{
  PROC
  CalIndexSet cis(set);
  for(unsigned long i=0; i<cis.size(); i++)
    addConstituent(cis.getConstituent(i));
  cis.close();
  ENDPROC
}

void CalIndex::save(const bool fullpath, const STime& analysisdate,
		    const STime& observationdate) const
{
  PROC
  CalIndexSet set(_name, _master, fullpath, _category, analysisdate, observationdate);
  for(CalIndex::const_iterator i = begin(); i != end(); i++)
    set.insert(*i);
  set.close();
  ENDPROC
}

void CalIndex::print(ostream & os) const
{
  PROC
  for(CalIndex::const_iterator i = begin(); i != end(); i++){
    if(errHandler.verbosity(Msg::AppLib) >= Msg::Verbose)
      os << **i << endl;
    else
      os << (*i)->name() << endl;
  }      
  ENDPROC
}
