//
// CalIndex -- XMM Science Analysis System
//
// Giuseppe Vacanti, XMM-SOC, European Space Agency
//   

#ifndef CalIndex_H
#define CalIndex_H

#include <string>
#include <set>
#include <Tracer.h>
#include "CcfConstituent.h"
#include <STime.h>

class CalIndex : public list<CcfConstituent *> {
 public:

  CalIndex(const string name, const bool master = false, const string category = "XMMCCF");
  virtual ~CalIndex() {}
  virtual void addConstituent(CcfConstituent * c);
  virtual bool removeConstituent(CcfConstituent * c);
  virtual bool removeConstituent(const string type, const string scope);
  virtual void addConstituentsFromSet(const string set);
  virtual void save(const bool fullpath = false, const STime& analysisdate = STime("now"),
		    const STime& observationdate = STime("now")) const;
  virtual string name() const;
  virtual string category() const;
  virtual void print(ostream & os) const;
  
 private:
  
  string _name, _category;
  bool _master, _exists;
  set<string, less<string> > _tally;

};

inline string CalIndex::name() const { PROC RETURN _name; }
inline string CalIndex::category() const { PROC RETURN _category; }

inline ostream & operator<<(ostream & os, const CalIndex & d)
{
  d.print(os);
  return os;
}


#endif


