//
// testcifbuildwitheval -- Giuseppe Vacanti
//
// Relies on the dates in the dummy constituents in mytestccf-with-endvaldate.
//
#include <string>
#include <Dal.h>
#include <cifbuild.h>
#include <param.h>
#include <metatask.h>
#include <CalIndexSet.h>
#include <cassert>
#include <errstr.h>

#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>

int main() {
  
  const string t1 = "1998-01-01T00:00:00";
  const string t2 = "1998-02-01T00:00:00";
  const string t3 = "1998-03-01T00:00:00";
  const string t4 = "1999-01-02T00:00:01"; // > EVALDATE(ABSCOEFF/3)
  const string cif = "ccf.cif";
  const string mif = "ccf.mif";
  const string dir = "mytestccf-with-endvaldate";

  // first pass
  cerr << "first pass" << endl;
  openParameters("cifbuild");
  setParameter("withccfpath", true);
  setParameter("ccfpath",dir.c_str());
  setParameter("withobservationdate",true);
  setParameter("analysisdate",t1);
  setParameter("calindexset", cif);
  setParameter("observationdate",t1);
  setParameter("category","XMMCCF");
  task(cifbuild);

  CalIndexSet * cis;
  cis = new CalIndexSet(cif);
  assert(cis->size() == 4);

  assert(cis->getConstituent(3)->type() == "CTI");
  assert(cis->getConstituent(3)->issue() == 1);
  assert(cis->getConstituent(0)->type() == "ABSCOEFS");
  assert(cis->getConstituent(0)->issue() == 3);
  cis->close();

  // check that the correct table name and attributes are there
  DataSet *set = dataSetServer->open(cif, Dal::Read);
  Table *tab = set->table("CALINDEX");
  tab->attribute("OBSVDATE");
  tab->attribute("ANALDATE");
  tab->column("VALDATE-END");
  dataSetServer->close(set);

  // second pass
  cerr << "second pass" << endl;
  setParameter("withobservationdate",true);
  setParameter("analysisdate","now");
  setParameter("observationdate",t4);
  task(cifbuild);

  cis = new CalIndexSet(cif);
  assert(cis->size() == 4);

  assert(cis->getConstituent(0)->type() == "ABSCOEFS");
  assert(cis->getConstituent(0)->issue() == 3);
  cis->close();


  // MIF: this is like the first pass, but we first go through a MIF
  cerr << "mif pass" << endl;
  openParameters("cifbuild");
  setParameter("withccfpath", true);
  setParameter("ccfpath", dir.c_str());
  setParameter("masterindex", true);
  setParameter("calindexset", mif);
  task(cifbuild);
  closeParameters();

  openParameters("cifbuild");
  setParameter("withmasterindexset", true);
  setParameter("masterindexset", mif);
  setParameter("withobservationdate",true);
  setParameter("analysisdate",t1);
  setParameter("calindexset", cif);
  setParameter("observationdate",t1);
  setParameter("category","XMMCCF");
  task(cifbuild);

  cis = new CalIndexSet(cif);
  assert(cis->size() == 4);

  assert(cis->getConstituent(0)->type() == "ABSCOEFS");
  assert(cis->getConstituent(0)->issue() == 3);
  cis->close();





  unlink(cif.c_str());
  unlink(mif.c_str());

}
  
